from random import randint, randrange
from browser import document, bind, ajax, aio, timer
from browser.local_storage import storage
import json


# Przyklad zastosowania klas i obiektów do zdefiniowania profilu użytkownika
# nie zdazylem dokonczyc generowania i wyswietlania profilow uzytkownikow, wiec
# ten jeden fragment kodu akurat nie jest wykorzystywany na stronie
class User:
    # konstruktor
    def __init__(self, fname, lname, gender, age):
        self.fname = fname
        self.lname = lname
        self.fullname = f'{fname} {lname}'
        self.gender = gender
        self.age = age

    def __str__(self):
        return f'User\n{self.fname=}\n{self.lname=}\n{self.fname=}\n{self.fullname=}\n{self.gender=}\n{self.age=}'

    # getter, setter i pola prywatne
    @property
    def photo(self): return self._photo

    @photo.setter
    def photo(self, url): self._photo = url

    # metoda dostepna wylacznie w klasie
    @classmethod
    def generate_random_user(cls):
        return cls(fname='random first name', lname='random last name', gender='random gender', age='random age')


# Przełączanie stylow dla aktywnych i nieaktywnych elementow
# przykład opcjonalnych i domyslnych argumentow
def toggle_active_class(evt=None,
                        element=None,
                        onOff=None,
                        cls='active'):

    element = element or evt.target
    class_list = element.class_name.split(' ')

    if cls in class_list and onOff != 'on':
        class_list.remove(cls)
    elif cls not in class_list and onOff != 'off':
        class_list.append(cls)

    element.class_name = ' '.join(class_list)


navbar_profile = document.select('.navbar-user-profile')[0]


@bind('.navbar-user-profile', 'click')
def toggle_profile_dropdown(evt):
    toggle_active_class(evt, element=navbar_profile)


@bind('.navbar-btn', 'click')
@bind('main', 'click')
@bind('footer', 'click')
def close_profile_dropdown_nofocus(evt):
    toggle_active_class(evt, element=navbar_profile, onOff='off')


# Inny sposob żeby napisać to samo co powyżej z petlami i list comprehensions
# for elements in [document.select(matching_elements)
#                  for matching_elements
#                  in ['.navbar-btn', 'main', 'footer']]:
#     for element in elements:
#         element.bind('click', lambda:
#                      toggle_active_class(
#                          element=navbar_profile,
#                          onOff='off'))


def update_favorite_in_localstorage(element):
    index = element.parent.parent.index()
    storage[f'auction_{index + 1}_is_favorite'] = str(
        'active' in element.class_name)


def toggle_favorite(evt):
    evt.preventDefault()
    toggle_active_class(evt)
    update_favorite_in_localstorage(evt.target)


favorite_buttons = document.select('.promoted-offer-star')

for btn in favorite_buttons:
    btn.bind('click', toggle_favorite)
    index = btn.parent.parent.index()
    try:
        if storage[f'auction_{index + 1}_is_favorite'] == 'True':
            toggle_active_class(element=btn, onOff='on')
    except KeyError:
        pass


@bind('.categories-flex a', 'click')
def ignoreCategories(evt): evt.preventDefault()


# Losowanie danych i zdjec samochodow do aukcji

is_car_data_loaded = False

try:
    is_car_data_loaded = storage['is_car_data_loaded'] == 'True'
except KeyError:
    print('Car data not previously loaded')

PIXABAY_API_KEY = '9324111-d9394a0e56f6eb524da75f703'
RANDOM_VEHICLE_URL = 'https://random-data-api.com/api/vehicle/random_vehicle'


def format_query_str(query_args):
    return'&'.join(query_args).replace(' ', '+')


def pixabay_query(search_term):
    query = format_query_str((f'q={search_term}',
                              'image_type=photo',
                              'orientation=horizontal',
                              'per_page=100',
                              'category=transportation',
                              'safesearch=true'
                              ))

    return f'https://pixabay.com/api/?key={PIXABAY_API_KEY}&{query}'


async def get_car_data_async(cb, args):
    # docstring
    '''
    funkcja ktora zajela zbyt dlugo do napisania wykorzystujaca:
    asynchronicznosc, domknięcia, rekurencje i rozpakowywanie slownikow

    losuje dane samochodu i zdjecie bliskie do nazwy wylosowanego samochodu
    zwraca dane i url zdjecia w formie łączonego slownika
    '''
    which_attempt = 0

    # przetwarza odpowiedz na get request po zdjecie samochodu i zwraca url zdjecia
    # lub wola sie ponownie z zmienionymi argumentami az bedzie w stanie zwrocic zdjecie
    async def handle_pixabay_results(request):
        nonlocal which_attempt
        which_attempt += 1
        response = request.data
        results_are_found = '[]' not in response

        if results_are_found:
            res_json = json.loads(response)
            result_count = len(res_json['hits'])
            img_index =\
                randrange(0, result_count - 1) if (result_count > 1) else 0
            car_image_url = res_json['hits'][img_index]['webformatURL']

            # debug print: dziwne rzeczy sie dzialy jesli chodzi o kolejnosc dzialan
            print(
                f'results found attempt {which_attempt-1}, {which_attempt=} {result_count=} {car_image_url=} {img_index=}')
            return car_image_url

        img_query_url = await aio.get(queries[which_attempt])
        return await handle_pixabay_results(img_query_url)

    # wczytuje dane losowego samochodu z random-data-api.com
    car_data_req = await aio.get(RANDOM_VEHICLE_URL)
    car_data_json = json.loads(car_data_req.data)

    car_name = car_data_json['make_and_model']
    car_make = car_name.split()[0]

    # to musi tu byc bo wyswietlalo zdjecia prezydentow
    if ('Lincoln' in car_name) or ('Lincoln' in car_make):
        car_name, car_make = 'Ford', 'Ford'

    # szuka zdjecia z api pixabay na podstawie pobranych danych
    # jesli nie znajdzie to szuka w kolejnosci:
    # najpierw po modelu i marce, potem samej marce potem dowolny samochod

    search_terms = (car_name, car_make, "car")
    queries = tuple(pixabay_query(s) for s in search_terms)

    pixabay_fut = aio.Future()
    img_query_url = await aio.get(queries[which_attempt])
    pixabay_fut.set_result(await handle_pixabay_results(img_query_url))

    result_url = {'car_img': await pixabay_fut}
    result_data = await {**car_data_json, **result_url, "args": {**args}}

    # przekazuje otrzymane dane do callbacku podanego w parametrach
    aio.run(cb(await result_data))


# Wyswietlanie danych i zdjec w promowanych aukcjach
def format_car_name(str):
    return str.replace('Chevy', 'Chevrolet')


async def set_offer_data(data):
    '''
    przeksztalca html dla ofert korzystajac z danych w przekazanym slowniku
    oraz zapisuje w localstorage numer aukcji jako klucz i dane w
    formie slownika przeksztalconego na str jako wartosc
    '''
    print('CB set_offer_data:', await data)

    elt = data['args']['element']
    title_element = elt.get(selector='.promoted-offer-title')[0]
    img_element = elt.get(selector='.promoted-offer-img')[0]
    which_auction = data['args']['which_auction']

    data_str = dict(data)
    del data_str['args']['element']
    data_str = json.dumps(data_str)
    storage[f'auction_{which_auction}'] = data_str

    car_name = format_car_name(data['make_and_model'])
    car_type = data['car_type']
    car_img = data['car_img']

    title_element.html = f'<span>{car_name}: {car_type}</span>'
    img_element.style = f'background-image: url({car_img});'

    storage['is_car_data_loaded'] = 'True'


def set_promoted_offers_prices():
    '''losuje ceny i zapisuje je w localstorage dla ponumerowanych aukcji'''
    for offer_index, price in enumerate(document.select('.promoted-offer-price-number')):
        val = []

        for i in range(6):
            num_min = 0 if i > 0 else 1
            num_max = 9 if i < 4 else 0
            num = randint(num_min, num_max)
            val.append(num)

        val = ''.join([str(num) for num in val])
        price.text = f'{val[0:3]} {val[3:6]}'
        storage[f'auction {offer_index + 1}_price'] = price.text


promoted_offers = document.select('.promoted-offer')


def clear_favorites():
    for btn in favorite_buttons:
        index = btn.parent.parent.index()
        try:
            if storage[f'auction_{index + 1}_is_favorite'] == 'True':
                toggle_active_class(element=btn, onOff='off')
                storage[f'auction_{index + 1}_is_favorite'] = 'False'
        except KeyError:
            pass


def load_offers(was_loaded=is_car_data_loaded):
    if was_loaded:
        for index, element in enumerate(promoted_offers):
            args = json.loads(storage[f'auction_{index + 1}'])
            args['args']['element'] = element
            aio.run(set_offer_data(args))
    else:
        for index, element in enumerate(promoted_offers):
            args = {'element': element, 'which_auction': index + 1}
            aio.run(get_car_data_async(set_offer_data, args))

    set_promoted_offers_prices()


@bind('.reload-btn', 'click')
def reload_offers(_evt):
    load_offers(was_loaded=False)
    clear_favorites()


load_offers()
